<?php

/**
 * Функции и классы для панели управления
 * скрипта MaRKeT Script.
 */
 
 

/**
 * Проверка и установка прав доступа на папки
 *
 * @return array
 */
 
function checkChmods() {

	$errors = array();
	
	// Папка администратора
	$admin_folder = MARKET_ADMIN_PATH;
	if(getChmod($admin_folder) != '0777')
		array_push($errors, array('code' => '777', 'dir' => $admin_folder));
		
	// Папка с данными
	$data_folder = MARKET_DATA_PATH;
	if(getChmod($data_folder) == '0777') {
	
		// Создание файла .htaccess
		$htaccess_path    = MARKET_DATA_PATH . '/.htaccess';
		if(!file_exists($htaccess_path)) {
			$htaccess_content = "order allow,deny\r\ndeny from all";
			$htaccess_handler = fopen($htaccess_path, 'w');
			chmod($htaccess_path, 0777);		
			fwrite($htaccess_handler, $htaccess_content);
			fclose($htaccess_handler);
		}
		
		// Создание папки с настройками
		if(!is_dir($data_folder . '/config'))
			mkdir($data_folder . '/config', 0777);
		
		// Создание папки с настройками биллингов
		if(!is_dir($data_folder . '/config/billings'))
			mkdir($data_folder . '/config/billings', 0777);
		
		// Создание папки с переменными
		if(!is_dir($data_folder . '/var'))
			mkdir($data_folder . '/var', 0777);
		
		// Создание папки с товарами
		if(!is_dir($data_folder . '/items'))
			mkdir($data_folder . '/items', 0777);
		
		// Создание папки с данными товаров
		if(!is_dir($data_folder . '/items/data'))
			mkdir($data_folder . '/items/data', 0777);
		
		// Создание папки с картинками товаров
		if(!is_dir($data_folder . '/items/images'))
			mkdir($data_folder . '/items/images', 0777);
		
		// Создание папки с файлами товаров
		if(!is_dir($data_folder . '/items/files'))
			mkdir($data_folder . '/items/files', 0777);
		
	} else
		array_push($errors, array('code' => '777', 'dir' => $data_folder));
	
	return $errors;
}



/**
 * Получение Chmod папки или файла
 *
 * @param str $path
 *
 * @return str
 */
 
function getChmod($path) {

	return substr(sprintf('%o', fileperms($path)), -4);
}



/**
 * Создание нового товара
 *
 * Создает новый товар и его файл данных,
 * картинку и скачиваемый файл.
 *
 * @param array $item
 *
 * @return boolen
 */
 
function createItem($item) {

	// Время
	$timestamp = time();
		
	// ID и hash
	if($item['id'] == false) {
	
		$item['id']   = getNewItemID();
		$item['hash'] = md5(realpath('.') . $timestamp . $item['name'] . $item['id']);		
	}
	
	// Загрузка файла товара
	if($item['file'] && $item['file']['tmp_name'] != '')
		$item['file'] = _uploadFile($item['file'], MARKET_ITEMS_FILES_PATH, false, $item['id']);
	else
		$item['file'] = $item['_file'];
	
	// Загрузка картинки товара
	if($item['image'] && $item['image']['tmp_name'] != '')
		$item['image'] = _uploadFile($item['image'], MARKET_ITEMS_IMAGES_PATH, false, $item['id']);
		
	// Подготовка названия товара
	$item['name']  = addslashes(mb_substr(htmlspecialchars(stripslashes(strip_tags($item['name']))), 0, MARKET_NAME_MAX_LENGTH));
		
	// Подготовка описания товара
	$item['descr'] = addslashes(mb_substr(htmlspecialchars(stripslashes(strip_tags($item['descr']))), 0, MARKET_DESCR_MAX_LENGTH));
		
	// Подготовка цены товара
	$item['price'] = formatPrice(htmlspecialchars(strip_tags($item['price'])));
	
	// Проверка перед записью данных
	if($item['id'] && $item['name'] != '' && $item['price'] != '') {

		// Создание файла данных
		$item_data_file_path    = MARKET_ITEMS_DATA_PATH . '/' . $item['id'] . '.php';
		$item_data_file_handler = fopen($item_data_file_path, 'w');
		
		// Установка прав доступа файла данных
		chmod($item_data_file_path, 0777);
		
		// Запись данных о товаре
		$item_data = "<?php\r\n\$item = array(\r\n\t";
		$item_data .= "'id'        => '". $item['id'] ."',\r\n\t";
		$item_data .= "'name'      => '". $item['name'] ."',\r\n\t";
		$item_data .= "'descr'     => '". $item['descr'] ."',\r\n\t";
		$item_data .= "'filename'  => '". $item['file']['name'] ."',\r\n\t";
		$item_data .= "'price'     => '". $item['price'] ."',\r\n\t";
		$item_data .= "'price_free'=> '". $item['price_free'] ."',\r\n\t";
		$item_data .= "'currency'  => '". $item['currency'] ."',\r\n\t";
		$item_data .= "'hash'      => '". $item['hash'] ."'\r\n\t";
		$item_data .= ");\r\n?>";
		
		fwrite($item_data_file_handler, $item_data);
		fclose($item_data_file_handler);
	}
}



/**
 * Загрузка файлов
 *
 * Загружает файл в указанную папку.
 * При необходимости переименовывает.
 *
 * @param array $file
 * @param str $folder
 * @param array $valid_types = false
 * @param str $rename = false
 *
 * @return str
 */

function _uploadFile($file, $folder, $valid_types = false, $rename = false) {
	
	// Расширение файла	
	$ext = substr($file['name'], 1 + strrpos($file['name'], '.'));

	// Конечное имя файла
	$filename = $rename != false ? $rename . '.' . $ext : $file['name'];
	
	// Уже имеющиеся файлы
	$_filename = glob($folder . '/' . $rename . '.*');
	
	if($_filename) {
		foreach($_filename as $_file) {
			if(file_exists($_file))
				unlink($_file);
		}
	}
	
	// Проверка на загружаемость файла
	if(is_uploaded_file($file['tmp_name'])) {
	
		// Проверка на валидность (если нужна)
		if($valid_types != false) {
			
			if (!in_array($ext, $valid_types))			
				return false;			
		}	
		
		// Загрузка
		if (move_uploaded_file($file['tmp_name'], $folder . '/' . $filename)) {
			
			// Установка прав доступа
			chmod($folder . '/' . $filename, 0777);
			return array('new_name' => $filename, 'name' => $file['name']);	
					
		} else			
			return false;
	}
}



/**
 * Удаление товара
 *
 * Удаляет информацию о товаре,
 * его картинку и файл.
 *
 * @param int $id (ID товара)
 *
 * @return boolen
 */
 
function deleteItem($id) {
	
	// Удаление данных о товаре
	$item_data = MARKET_ITEMS_DATA_PATH . '/' . $id . '.php';
	if(file_exists($item_data))
		unlink($item_data);
	
	// Удаление файла товара
	$item_file = glob(MARKET_ITEMS_IMAGES_PATH . '/' . $id . '.*');
	if(file_exists($item_file[0]))
	    unlink($item_file[0]);
	
	// Удаление файла товара
	$item_image = glob(MARKET_ITEMS_FILES_PATH . '/' . $id . '.*');
	if(file_exists($item_image[0]))
	    unlink($item_image[0]);
	    
	return true;
}



/**
 * Сохранение настроек биллингов
 *
 * @param array $data
 */
 
function saveBillings($billings_data) {
	
	// Для каждого биллинга
	foreach($billings_data as $billing => $data) {
	
		$code = "";
		$size = sizeof($data);
		$i    = 1;
		
		// Сохранение настроек
		foreach($data as $param => $value) {
		
			$value = trim(htmlspecialchars(strip_tags($value)));
			
			if($value != '')
				$code .= "\t'" . $param . "' => '" . $value . "'" . ($i != $size ? "," : "") . "\r\n";
				
			$i ++;
		}	
		if($code != '')
			$code = "<?php\r\n\$_BILL = array(\r\n" . $code . ");\r\n?>";
		
		// Создание файла данных
		$billing_data_file_path    = MARKET_DATA_PATH . '/config/billings/' . $billing . '.php';
		$billing_data_file_handler = fopen($billing_data_file_path, 'w');
		
		// Установка прав доступа файла данных
		chmod($billing_data_file_path, 0777);
		
		// Запись в файл
		if($code != '')
			fwrite($billing_data_file_handler, $code);
			
		fclose($billing_data_file_handler);
	}
}



/**
 * Получение логина авторизации
 *
 * @return str
 */
 
function getAuthData() {

	$login    = '';
	
	// Чтение файла .htpasswd
	$htpasswd = MARKET_ADMIN_PATH . '/.htpasswd';
	if(file_exists($htpasswd)) {
		$handle   = fopen($htpasswd, "r");
		$contents = fread($handle, filesize($htpasswd));	
		fclose($handle);
		
		// Получение логина
		$log_pass = explode(':', $contents);
		$login    = $log_pass[0];
	}
	
	// Получение эл. почты администратора
	if(defined('MARKET_EMAIL_ADMIN'))
		$email = MARKET_EMAIL_ADMIN;
	else
		$email = '';
	
	// Получение кода GA
	if(defined('MARKET_GA_CODE'))
		$ga_code = stripslashes(MARKET_GA_CODE);
	else
		$ga_code = '';
	
	return array('login' => $login, 'email' => $email, 'ga_code' => $ga_code);
}



/**
 * Сохранение логина и пароля .htpasswd
 *
 * @param array $auth
 */
 
function saveAuthData($auth) {	
	
	// Сохранение кода Google Analytics
	saveVar('MARKET_GA_CODE', $auth['ga_code']);

	// Обработка логина и пароля
	foreach($auth as $param => $value) {		
		
		$value = trim(htmlspecialchars(strip_tags($value)));
		$auth[$param] = $value;
	}
	
	// Если логин и пароли непусты и совпадают
	if($auth['login'] != '' && $auth['pass'] != '' && $auth['pass'] == $auth['pass2']) {
	
		// Создание файла .htaccess для панели управления
		$htaccess_content = "AuthType Basic\r\n" . "AuthName \"MaRKeT Script Admin Panel\"\r\n" . "AuthUserFile " . MARKET_ADMIN_PATH . "/.htpasswd\r\n" . "Require valid-user";
		$htaccess_path    = MARKET_ADMIN_PATH . '/.htaccess';
		$htaccess_handler = fopen($htaccess_path, 'w');
		chmod($htaccess_path, 0777);		
		fwrite($htaccess_handler, $htaccess_content);
		fclose($htaccess_handler);
		
		// Кодирование пароля
		$auth['encode_pass'] = encodePass($auth['pass']);
		
		// Запись логина и пароля в файл .htpasswd
		$htpasswd_path    = MARKET_ADMIN_PATH . '/.htpasswd';
		$htpasswd_handler = fopen($htpasswd_path, 'w');
		fwrite($htpasswd_handler, $auth['login']. ':' . $auth['encode_pass']);			
		fclose($htpasswd_handler);
	}
	
	// Сохранение эл. почты
	saveVar('MARKET_EMAIL_ADMIN', $auth['email']);
}
 
 
 
/**
 * Создание пароля для .htpasswd
 *
 * @param str $pass
 *
 * @return str $pass
 */

function encodePass($pass) {

    $pass = crypt(trim($pass), base64_encode(CRYPT_STD_DES));
    return $pass;
}
?>