<?php

/**
 * Функции и классы для работы с товарами
 */
 

/**
 * Получение списка товаров
 *
 * @return array() - массив с товарами (id, name, descr, filename, image, price)
 */
 
function getItems() {
	
	$items = array();
	
	if($handle = opendir(MARKET_ITEMS_DATA_PATH)) {
	
		while (false !== ($file = readdir($handle))) {
	        
	        if($file != '.' && $file != '..') {
	        	
	        	$item = getItem(basename($file, '.php'));
	        	array_push($items, $item);	        	
	        }	        
	    }		
	}
	
	array_multisort($items, SORT_DESC);
	
	return $items;
}



/**
 * Получение информации о товаре
 *
 * @param int $id
 *
 * @return array
 */
 
function getItem($id) {
	
	// Проверка на существование товара
	$file = MARKET_ITEMS_DATA_PATH . '/' . $id . '.php';
	if(file_exists($file)) {
	
		// Получение данных
		include($file);
	
		// Получение картинки товара
		$item_image = glob(MARKET_ITEMS_IMAGES_PATH . '/' . $item['id'] . '.*');		
		if(isset($item_image[0])) {
		
			$item['image']      = basename($item_image[0]);
			$item['image_url']  = MARKET_ROOT_URL . '/?image=' . $item['id'];
			$item['image_path'] = $item_image[0];
		}
		
		else {
		
			$item['image']      = false; 
			$item['image_url']  = false;
			$item['image_path'] = false;
		}
		
		$item['price_fr']   = isset($item['price_free']) ? $item['price_free'] : '0';
		$item['price_free'] = $item['price_fr'];
		
		return $item;
	
	} else
		return false;
}



/**
 * Получение ID нового товара
 *
 * @return int
 */
 
function getNewItemID() {
	
	$newID = 0;
	
	if($handle = opendir(MARKET_ITEMS_DATA_PATH)) {
	
		while (false !== ($file = readdir($handle))) {
	        
	        if($file != '.' && $file != '..') {
	        
	        	$info      = pathinfo($file);
				$file_name = basename($file, '.' . $info['extension']);
	        	
	        	if($file_name > $newID)
	        		$newID = $file_name;	        	
	        }	        
	    }		
	}
	
	$newID ++;
	
	return $newID;	
}



/**
 * Форматирование стоимости товара
 *
 * @param float $price
 * @param int $decimals = 2
 *
 * @return float
 */
 
function formatPrice($price, $decimals = 2) {
	
	return number_format($price, $decimals, '.', '');
}



/**
 * Отображение стоимости товара
 *
 * @param float $price
 * @param int $currency
 * @param int $decimals = 2
 *
 * @return str
 */
 
function displayPrice($price, $currency, $decimals = 2) {
	
	// Получение списка валют
	$currencies_lc = unserialize(MARKET_CURRENCIES_LABELS_C);
	
	// Получение текущей валюты
	$currency_str = $currencies_lc[$currency];
	
	// Формирование цены
	$price = formatPrice(floatval($price));
	
	// Значок валюты
	$price_str = $currency == 0 ? $price . '&nbsp;' . $currency_str : $currency_str . $price;
	
	return $price_str;
}



/**
 * Отображение размера файла
 *
 * @param int $size
 * @param int $precision
 *
 * @return str
 */
 
function displayFileSize($size, $precision = 1) {

	$size  = round(abs($size));
	$units = unserialize(MARKET_FILESIZES_LABELS);
	
	$file_b  = $size;
	$file_kb = round(($size / 1024), 2);
	$file_mb = round(($size / 1048576), 2);
	$file_gb = round(($size / 1073741824), 2);
	
	$size_   = $file_b;
	$unit    = 0;
	
	if($file_b > 1000) {
		$size_ = $file_kb;
		$unit  = 1;
	}
	
	if($file_kb > 1000) {
		$size_ = $file_mb;
		$unit  = 2;
	}
	
	if($file_mb > 1000) {
		$size_ = $file_gb;
		$unit  = 3;
	}
		
	return $size_ . ' ' . $units[$unit];
}



/**
 * Скачивание файла
 *
 * Функция скачивает файл,
 * если его ID и hash верны.
 *
 * @param int $id
 * @param str $hash
 */
 
function downloadItem($id, $hash) {

	// Получение данных о файле
	$item = getItem($id);
	
	// Проверка hash файла
	if($hash == $item['hash'] && $id != '' && $hash != '') {
		
		$file         = glob(MARKET_ITEMS_FILES_PATH . '/' . $item['id'] . '.*');
		$file['file'] = $file[0];
		$file['name'] = $item['filename'];
		
		// Проверка наличия файла
		if(file_exists($file['file'])) {
			
			downloadFile($file['file'], $file['name'], 4000);
		}
	}	
}



/**
 * Скачивание файла с поддержкой
 * докачки, регулирования скорости
 * и потоковой отдачи
 */

function downloadFile($fileLocation, $fileName, $maxSpeed = 1000, $doStream = false) { 
    if (connection_status()!=0) return(false); 
    $extension = strtolower(end(explode('.',$fileName))); 

    
    $fileTypes['swf'] = 'application/x-shockwave-flash'; 
    $fileTypes['pdf'] = 'application/pdf'; 
    $fileTypes['exe'] = 'application/octet-stream'; 
    $fileTypes['zip'] = 'application/zip'; 
    $fileTypes['doc'] = 'application/msword'; 
    $fileTypes['xls'] = 'application/vnd.ms-excel'; 
    $fileTypes['ppt'] = 'application/vnd.ms-powerpoint'; 
    $fileTypes['gif'] = 'image/gif'; 
    $fileTypes['png'] = 'image/png'; 
    $fileTypes['jpeg'] = 'image/jpg'; 
    $fileTypes['jpg'] = 'image/jpg'; 
    $fileTypes['rar'] = 'application/rar';     
     
    $fileTypes['ra'] = 'audio/x-pn-realaudio'; 
    $fileTypes['ram'] = 'audio/x-pn-realaudio'; 
    $fileTypes['ogg'] = 'audio/x-pn-realaudio'; 
     
    $fileTypes['wav'] = 'video/x-msvideo'; 
    $fileTypes['wmv'] = 'video/x-msvideo'; 
    $fileTypes['avi'] = 'video/x-msvideo'; 
    $fileTypes['asf'] = 'video/x-msvideo'; 
    $fileTypes['divx'] = 'video/x-msvideo'; 

    $fileTypes['mp3'] = 'audio/mpeg'; 
    $fileTypes['mp4'] = 'audio/mpeg'; 
    $fileTypes['mpeg'] = 'video/mpeg'; 
    $fileTypes['mpg'] = 'video/mpeg'; 
    $fileTypes['mpe'] = 'video/mpeg'; 
    $fileTypes['mov'] = 'video/quicktime'; 
    $fileTypes['swf'] = 'video/quicktime'; 
    $fileTypes['3gp'] = 'video/quicktime'; 
    $fileTypes['m4a'] = 'video/quicktime'; 
    $fileTypes['aac'] = 'video/quicktime'; 
    $fileTypes['m3u'] = 'video/quicktime'; 

    $contentType = $fileTypes[$extension]; 
     
     
    header("Cache-Control: public"); 
    header("Content-Transfer-Encoding: binary\n"); 
    header('Content-Type: $contentType'); 

    $contentDisposition = 'attachment'; 
     
    if($doStream == true) { 
        $array_listen = array('mp3','m3u','m4a','mid','ogg','ra','ram','wm', 
        'wav','wma','aac','3gp','avi','mov','mp4','mpeg','mpg','swf','wmv','divx','asf'); 
        if(in_array($extension,$array_listen)){  
            $contentDisposition = 'inline'; 
        } 
    } 

    if (strstr($_SERVER['HTTP_USER_AGENT'], "MSIE")) { 
        $fileName= preg_replace('/\./', '%2e', $fileName, substr_count($fileName, '.') - 1); 
        header("Content-Disposition: $contentDisposition; filename=\"$fileName\""); 
    } else { 
        header("Content-Disposition: $contentDisposition; filename=\"$fileName\""); 
    } 
     
    header("Accept-Ranges: bytes");    
    $range = 0; 
    $size = filesize($fileLocation); 
     
    if(isset($_SERVER['HTTP_RANGE'])) { 
        list($a, $range)=explode("=",$_SERVER['HTTP_RANGE']); 
        str_replace($range, "-", $range); 
        $size2=$size-1; 
        $new_length=$size-$range; 
        header("HTTP/1.1 206 Partial Content"); 
        header("Content-Length: $new_length"); 
        header("Content-Range: bytes $range$size2/$size"); 
    } else { 
        $size2=$size-1; 
        header("Content-Range: bytes 0-$size2/$size"); 
        header("Content-Length: ".$size); 
    } 
         
    if ($size == 0 ) exit; 
    @set_magic_quotes_runtime(0);  
    $fp=fopen("$fileLocation","rb"); 
     
    fseek($fp,$range); 
   
    while(!feof($fp) and (connection_status()==0)) { 
        @set_time_limit(0); 
        print(fread($fp,1024*$maxSpeed)); 
        flush(); 
        ob_flush(); 
        sleep(1); 
    } 
    fclose($fp); 
            
    return((connection_status()==0) and !connection_aborted()); 
}



/**
 * Добавление кода Ecommerce
 * для Google Analytics
 *
 * @param array $item
 *
 * @return str
 */
 
function addGAEcommerceCode($item) {

	$code = '';
	
	// Проверка на наличие кода
	if(defined('MARKET_GA_CODE') && MARKET_GA_CODE != '' && $item) {
	
		// Проверка на оплаченность товара
		if(isset($_SESSION['GA_DATA_'. $item['id'] . '_' . md5(MARKET_ROOT_PATH)])) {
		
			// Данные о покупке
			$_GA  = $_SESSION['GA_DATA_'. $item['id'] . '_' . md5(MARKET_ROOT_PATH)];
			
			// Определение версии кода GA
			if(mb_strpos(MARKET_GA_CODE, '_gaq') > 0) { 
			
				// Новая версия
				$ecommerce_code = "_gaq.push(['_trackPageview']);\r\n" .
					"_gaq.push(['_addTrans', " .
					"'" . $_GA['trans_id'] . "', " .
					"'" . $_GA['store_name'] . "', " .
					"'" . $_GA['total_price'] . "', " .
					"'', '', '', '', ''" .
					"]);\r\n\r\n" .
					"_gaq.push(['_addItem', " .
					"'" . $_GA['trans_id'] . "', " .
					"'" . $_GA['item_id'] . "', " .
					"'" . $_GA['item_name'] . "', " .
					"'', " .
					"'" . $_GA['item_price'] . "', " .
					"'" . $_GA['item_quant'] . "' " .
					"]);\r\n" .
					"_gaq.push(['_trackTrans']);\r\n";
					
				$code    = str_replace("_gaq.push(['_trackPageview']);", $ecommerce_code, stripslashes(MARKET_GA_CODE));		
			}
			
			elseif(mb_strpos(MARKET_GA_CODE, 'pageTracker') > 0) {
			
				// Старая версия
				$ecommerce_code = "pageTracker._trackPageview();\r\n" .
					"pageTracker._addTrans( " .
					"'" . $_GA['trans_id'] . "', " .
					"'" . $_GA['store_name'] . "', " .
					"'" . $_GA['total_price'] . "', " .
					"'', '', '', '', '' " .
					");\r\n\r\n" .
					"pageTracker._addItem(" .
					"'" . $_GA['trans_id'] . "', " .
					"'" . $_GA['item_id'] . "', " .
					"'" . $_GA['item_name'] . "', " .
					"'', " .
					"'" . $_GA['item_price'] . "', " .
					"'" . $_GA['item_quant'] . "' " .
					");\r\n" .
					"pageTracker._trackTrans();\r\n";
					
				$code    = str_replace("pageTracker._trackPageview();", $ecommerce_code, stripslashes(MARKET_GA_CODE));
			}
		}		
		
		else $code = stripslashes(MARKET_GA_CODE);
	}
	
	return $code;
}



/**
 * Конвертер валют (рубли, доллары, евро)
 * 
 * Конвертирует рубли, доллары и евро
 * друг в друга по курсу ЦБ РФ
 *
 * @param float $val
 * @param str $from
 * @param str $to
 * @param boolen $format = true
 *
 * @return float
 */
 
function convertPrice($val, $from, $to, $format = true) {
	 
	// Получаем текущие курсы валют с сайта www.cbr.ru 
	$content = fetchURL($link = "http://www.cbr.ru/scripts/XML_daily.asp?date_req=". date("d/m/Y") );
	
	// Разбираем содержимое, при помощи регулярных выражений 
	$pattern = "#<Valute ID=\"([^\"]+)[^>]+>[^>]+>([^<]+)[^>]+>[^>]+>[^>]+>[^>]+>[^>]+>[^>]+>([^<]+)[^>]+>[^>]+>([^<]+)#i"; 
	preg_match_all($pattern, $content, $out, PREG_SET_ORDER); 
	$dollar = ""; 
	$euro   = ""; 
	foreach($out as $cur) { 
	
		if($cur[2] == 840) $dollar = str_replace(",",".",$cur[4]);			 
		if($cur[2] == 978) $euro   = str_replace(",",".",$cur[4]); 
	} 
	
	// Матрица конвертации
	$multipliers = array(
		'RUB' => array(
			'RUB' => 1,
			'USD' => 1 / $dollar,
			'EUR' => 1 / $euro
		),
		'USD' => array(
			'USD' => 1,
			'RUB' => $dollar,
			'EUR' => $dollar / $euro
		),
		'EUR' => array(
			'EUR' => 1,
			'RUB' => $euro,
			'USD' => $euro / $dollar
		)
	);
	
	// Конвертация
	$val_ = $val * $multipliers[$from][$to];

	return $format ? formatPrice($val_) : $val_;
}



/**
 * Получение содержимого
 * удаленного файла
 *
 * @param str $url
 *
 * @return str
 */
 
function fetchURL($url) {
	
    $url_parsed = parse_url($url);
    $host = $url_parsed["host"];
    @$port = $url_parsed["port"];
    if ($port==0)
        $port = 80;
    $path = $url_parsed["path"];
    if ($url_parsed["query"] != "")
        $path .= "?".$url_parsed["query"];

    $out = "GET $path HTTP/1.0\r\nHost: $host\r\n\r\n";

    $fp = fsockopen($host, $port, $errno, $errstr, 30);

    fwrite($fp, $out);
    $body = false;
    while (!feof($fp)) {
        $s = fgets($fp, 1024);
        if ( $body )
            @$in .= $s;
        if ( $s == "\r\n" )
            $body = true;
    }
   
    fclose($fp);
   
    return $in;
}



/**
 * Отправка HTML писем
 */

function send_mime_mail($name_from, $email_from, $name_to, $email_to, $data_charset, $send_charset, $subject, $body, $copies) {

	if(strpos($email_to, ',') > 0) {
		$to = '';
		$et = explode(',', $email_to);
		for($i = 0; $i < sizeof($et); $i ++) {
			$to .= trim($et[$i]) . ', ';
		}
	} else
		$to 	= mime_header_encode($name_to, $data_charset, $send_charset) . ' <' . $email_to . '>';
	
	$subject 	= mime_header_encode($subject, $data_charset, $send_charset);
	$from 		= mime_header_encode($name_from, $data_charset, $send_charset) .' <' . $email_from . '>';
	if($data_charset != $send_charset) {
		$body = @iconv($data_charset, $send_charset . '//IGNORE', $body);
	}
	
	$headers 	= "From: ". $from ."\r\n";
	$headers   .= "Content-type: text/html; charset=". $send_charset ."\r\n";
	
	if($copies != '') {
		$em = explode('::', $copies);
		$pr = sizeof($em) - 1;
		for ($i = 0; $i < $pr; $i ++) {
			$headers .= 'Bcc: '.$em[$i] . "\r\n";
		}
	}

	return mail($email_to, $subject, $body, $headers);
}



/**
 * Кодировка заголовков писем
 */

function mime_header_encode($str, $data_charset, $send_charset) {

	if($data_charset != $send_charset)
		$str = @iconv($data_charset, $send_charset . '//IGNORE', $str);
	
	return '=?' . $send_charset . '?B?' . base64_encode($str) . '?=';
}



/**
 * Получение настроек всех биллингов
 *
 * @return array
 */
 
function getBillingsConf() {
	
	$confs = array();
	$dir   = MARKET_DATA_PATH . '/config/billings';
		
	if(is_dir($dir)) {
	
		$handle = opendir($dir);
	
		while (false !== ($file = readdir($handle))) {
	        
	        if($file != '.' && $file != '..') {
	        	
	        	include(MARKET_DATA_PATH . '/config/billings/' . $file);
	        	$confs[basename($file, '.php')] = $_BILL;        	
	        }	        
	    }		
	}
	
	return $confs;
}



/**
 * Сохранение переменных
 *
 * Сохраняет переменные в папку /data/var
 *
 * @param str $name
 * @param str $value
 * @param str $type = 'const'
 */
 
function saveVar($name, $value, $type = 'var') {
	
	$varfile  = MARKET_DATA_PATH . '/var/' . $type . '.' . strtolower($name) . '.php';
	$value    = addslashes(stripslashes($value));
	$content  = "<?php\r\n";
	$content .= $type == 'var' ? "define(\"" . $name . "\", \"" . $value . "\");" : "\$" . $name . " = \"" . $value . "\";";
	$content .= "\r\n?>";
	
	if(is_dir(MARKET_DATA_PATH . '/var')) {
		$handler  = fopen($varfile, 'w');	
		fwrite($handler, $content);			
		fclose($handler);
	}
}



/**
 * Генерация квадратных картинок
 * заданного размера из исходной
 *
 * @param str $src
 * @param int $arm
 */

function imageThumb($src, $arm) {

	// Размер и mime-type изображения
	$size = getimagesize($src);
	if ($size === false)
		return false;
	
	// Расширение
	$ext          = strtolower(substr($size['mime'], strpos($size['mime'], '/') + 1));
	$imagecreate_ = "imagecreatefrom" . $ext;
	$imagemime_   = "image" . $ext;
	$source_image = $imagecreate_($src);
	
	// Картинка-заготовка
	$result_image = imagecreatetruecolor($arm, $arm);
	
	// Размеры и пропорции	
	$min_dim = $size[0] > $size[1] ? $size[1] : $size[0];	
	$left    = ($size[0] - $min_dim) / 2;
	$top     = ($size[1] - $min_dim) / 2;
	$width   = $size[0]  - $left * 2;
	$height  = $size[1]  - $top * 2;

	// Создание итоговой картинки
	imagecopyresampled($result_image, $source_image, 0, 0,	$left, $top, $arm, $arm, $width, $height);	
	
	// Показ картинки
	header('Content-type: ' . $size['mime']);
	$imagemime_($result_image);	
	
	// Очистка памяти
	imagedestroy($result_image);
}



/**
 * Проверка на факт покупки товара
 *
 * @param array $item Переменная, в которой хранятся данные товара.
 */

function isAlreadyBought($item) {
	if(isset($_SESSION['already_bought']) && in_array($item['hash'], $_SESSION['already_bought'])) 
		return true;
	else 
		return false;
}



/**
 * Запоминание, что этот товар уже куплен
 * 
 * @param type $item 
 */

function setAlreadyBought($item) {
	
	$already_bought = isset($_SESSION['already_bought']) ? $_SESSION['already_bought'] : array();
	array_push($already_bought, $item['hash']);	
	$_SESSION['already_bought'] = array_unique($already_bought);
}



/**
* Провереяем валидность email адреса.
*/

function isValidEmail($email) {
	// First, we check that there's one @ symbol, 
	// and that the lengths are right.
	if (!ereg("^[^@]{1,64}@[^@]{1,255}$", $email)) {
		// Email invalid because wrong number of characters 
		// in one section or wrong number of @ symbols.
		return false;
	}
	// Split it into sections to make life easier
	$email_array = explode("@", $email);
	$local_array = explode(".", $email_array[0]);
	for ($i = 0; $i < sizeof($local_array); $i++) {
		if
	(!ereg("^(([A-Za-z0-9!#$%&'*+/=?^_`{|}~-][A-Za-z0-9!#$%&
	↪'*+/=?^_`{|}~\.-]{0,63})|(\"[^(\\|\")]{0,62}\"))$",
	$local_array[$i])) {
		  return false;
		}
	}
	// Check if domain is IP. If not, 
	// it should be valid domain name
	if (!ereg("^\[?[0-9\.]+\]?$", $email_array[1])) {
		$domain_array = explode(".", $email_array[1]);
		if (sizeof($domain_array) < 2) {
		    return false; // Not enough parts to domain
		}
		for ($i = 0; $i < sizeof($domain_array); $i++) {
		  if
	(!ereg("^(([A-Za-z0-9][A-Za-z0-9-]{0,61}[A-Za-z0-9])|
	↪([A-Za-z0-9]+))$",
	$domain_array[$i])) {
		    return false;
		  }
		}
	}
	return true;
}



/**
 * Отправка письма из мерчанта
 * для информирования владельца о покупке 
 * 
 */

function sendMerchantEmail() {
	
	// Информирование владельца о покупке
	if(defined('MARKET_EMAIL_ADMIN') && isValidEmail(MARKET_EMAIL_ADMIN))
		send_mime_mail('MaRKeT Script', 'noreply@marketscript.ru', MARKET_EMAIL_ADMIN, MARKET_EMAIL_ADMIN, 'utf-8', 'koi8-r', MARKET_MARKET_EMAIL_SUBJECT, MARKET_MARKET_EMAIL_BODY, '');
}



/**
 * Успешное проведение платежа
 * составление данных для Google Analytics
 * и редирект.
 * 
 * @param type $item
 * @param type $item_price
 * @param type $item_currency
 * @param type $item_id 
 */

function successPayment($item, $item_price, $item_currency, $item_id, $_BILL) {
	
	//Установка в сессии данных о том, что товар уже куплен.
	setAlreadyBought($item);

	$_BILL['GA_price'] = convertPrice($item_price, $item_currency, MARKET_GA_PRICE);

	// Составление данных для Google Analytics
	$_GA = array(
		'trans_id'    => $_BILL['inv_id'],
		'store_name'  => 'MaRKeT Script',
		'total_price' => $_BILL['GA_price'],

		'item_id'     => $item['id'],
		'item_name'   => $item['name'],
		'item_price'  => $_BILL['GA_price'],
		'item_quant'  => 1
	);
	$_SESSION['GA_DATA_' . $item['id'] . '_' . md5(MARKET_ROOT_PATH)] = $_GA;

	// Переход на страницу товара
	header('Location: ./?buy=' . $item_id, true, 301);			
}
?>
